import socket

hostname = socket.gethostname()

import logging
import sys

logging.basicConfig(stream=sys.stdout, level=logging.ERROR, format='%(message)s')

from ahisto_named_entity_search.entity import load_entities, Place, Person

all_entities = load_entities()
place_entities = [entity for entity in all_entities if isinstance(entity, Place)]
person_entities = [entity for entity in all_entities if isinstance(entity, Person)]

from ahisto_named_entity_search.entity import (Patchset, EntityType, patch_entities,
                                               default_entity_map_persons as entity_map_persons,
                                               default_entity_map_places as entity_map_places)

all_canonicalized_entities = {
    **patch_entities(Patchset('patchset-persons.xlsx', person_entities), person_entities, entity_map_persons),
    **patch_entities(Patchset('patchset-places.xlsx', place_entities), place_entities, entity_map_places),
}

unique_canonical_entities = sorted(set(all_canonicalized_entities.values()))
print(f'We selected {len(unique_canonical_entities)} canonical entities.')

numbers_of_cpus = {
    'apollo.fi.muni.cz': 80,
    'aura.fi.muni.cz': 256,
    'adonis.fi.muni.cz': 64,
    'mir': 32,
    'epimetheus1': 32,
    'epimetheus2': 32,
    'epimetheus3': 32,
    'epimetheus4': 32,
}
number_of_cpus = numbers_of_cpus[hostname]
total_weight = sum(numbers_of_cpus.values())

sliced_unique_canonical_entities = dict()
previous_boundary, next_boundary = 0.0, 0.0
for current_hostname, weight in sorted(numbers_of_cpus.items()):
    next_boundary += 1.0 * weight / total_weight
    from_index = int(round(previous_boundary * len(unique_canonical_entities)))
    to_index = int(round(next_boundary * len(unique_canonical_entities)))
    sliced_unique_canonical_entities[current_hostname] = unique_canonical_entities[from_index:to_index]
    previous_boundary = next_boundary


from ahisto_named_entity_search.document import Document, load_documents

documents = load_documents()

from ahisto_named_entity_search.index import FuzzyRegexIndex

from more_itertools import chunked

chunk_size = 100

from json import JSONDecodeError

import ahisto_named_entity_search.search.search
from ahisto_named_entity_search.search import Search
from ahisto_named_entity_search.search import SearchResultList

from typing import Iterable, TypeVar

T = TypeVar('T')

def silent_tqdm(iterable: Iterable[T], total: str, desc: str) -> Iterable[T]:
    return iterable

ahisto_named_entity_search.search.search.tqdm = silent_tqdm

print()
print()

from math import ceil
import re

from tqdm import tqdm

short_hostname = re.sub(r'\..*', '', hostname)

iterable = sliced_unique_canonical_entities[hostname]
iterable_size = int(ceil(len(iterable) / chunk_size))
iterable = enumerate(chunked(iterable, chunk_size))
iterable = tqdm(iterable, total=iterable_size, desc='Processing chunks')

for chunk_number, entities in iterable:
    filename = f'{short_hostname}-{chunk_number:03}'
    try:
        _ = SearchResultList.load(filename, entities)
        print(f'\n\nLoaded {filename}')
    except (IOError, JSONDecodeError):
        fuzzy_regex_results = Search(FuzzyRegexIndex(documents.values())).search(entities)
        fuzzy_regex_results.save(filename)
